/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.facade.def.impl;

import org.ow2.orchestra.facade.def.ProcessDefinition;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;

/**
 *
 * @author Gael Lalire
 */
public class ProcessDefinitionImpl extends DefinitionRecordImpl implements ProcessDefinition {

  private static final long serialVersionUID = 35906774703995610L;

  protected ActivityDefinitionUUID enclosedActivityUUID;

  protected ProcessDefinitionImpl() { }

  protected ProcessDefinitionImpl(final ProcessDefinitionUUID uuid,
      final String nameSpace, final String name) {
    super(uuid, "process", name);
  }

  public ProcessDefinitionImpl(final ProcessDefinition src) {
    super(src);
    enclosedActivityUUID = src.getEnclosedActivityUUID();
  }

  @Override
  public boolean equals(final Object obj) {
    if (obj == null) {
      return false;
    }
    if (!(obj.getClass().equals(this.getClass()))) {
      return false;
    }
    ProcessDefinitionImpl other = (ProcessDefinitionImpl) obj;
    return other.getUUID().equals(getUUID());
  }

  public ProcessDefinitionUUID getUUID() {
    return getProcessDefinitionUUID();
  }

  @Override
  public int hashCode() {
    return getUUID().hashCode();
  }

  /**
   * @return the enclosedActivityUUID
   */
  public ActivityDefinitionUUID getEnclosedActivityUUID() {
    return enclosedActivityUUID;
  }

  /**
   * @param enclosedActivityUUID the enclosedActivityUUID to set
   */
  public void setEnclosedActivityUUID(final ActivityDefinitionUUID enclosedActivityUUID) {
    this.enclosedActivityUUID = enclosedActivityUUID;
  }

}
