/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.facade.runtime.impl;

import java.util.HashSet;
import java.util.Set;

import org.ow2.orchestra.facade.runtime.ActivityInstance;
import org.ow2.orchestra.facade.uuid.ActivityInstanceUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;

/**
 *
 * @author Gael Lalire
 */
public class ActivityInstanceImpl extends RuntimeRecordImpl implements ActivityInstance {

  private static final long serialVersionUID = 9188864525348045460L;

  /**
   * Warning this field is not persisted so you have to use getter
   * instead of direct reference.
   */
  protected Set<ActivityInstanceUUID> enclosedActivitiesUUID;

  protected ActivityInstanceUUID uuid;

  protected ActivityInstanceImpl() {
  }

  protected ActivityInstanceImpl(final ActivityInstanceUUID uuid,
      final ProcessDefinitionUUID processUUID,
      final ProcessInstanceUUID instanceUUID) {
    super(processUUID, instanceUUID);
    this.uuid = uuid;
    enclosedActivitiesUUID = new HashSet<ActivityInstanceUUID>();
  }

  public ActivityInstanceImpl(final ActivityInstance record) {
    super(record);
    this.uuid = record.getUUID();
    enclosedActivitiesUUID = new HashSet<ActivityInstanceUUID>(record.getEnclosedActivitiesUUID());
  }

  @Override
  public String toString() {
    String st = this.getClass().getName()
      + "[uuid: " + getUUID()
      + ", processDefinitionUUID: " + getProcessDefinitionUUID()
      + ", processUUID: " + getProcessInstanceUUID()
      + ", startedDate: " + getStartedDate()
      + ", endedDate: " + getEndedDate();
    st += "]\n";
    return st;
  }

  public ActivityInstanceUUID getUUID() {
    return uuid;
  }

  public Set<ActivityInstanceUUID> getEnclosedActivitiesUUID() {
    return enclosedActivitiesUUID;
  }

  public void addEnclosedActivityUUID(final ActivityInstanceUUID activityInstanceUUID) {
    getEnclosedActivitiesUUID().add(activityInstanceUUID);
  }

  @Override
  public int hashCode() {
    return super.hashCode();
  }

  @Override
  public boolean equals(final Object obj) {
    if (obj == null) {
      return false;
    }
    if (!(obj.getClass().equals(this.getClass()))) {
      return false;
    }
    if (super.equals(obj)) {
      ActivityInstanceImpl other = (ActivityInstanceImpl) obj;
      return getEnclosedActivitiesUUID().equals(other.getEnclosedActivitiesUUID()) && uuid.equals(other.uuid);
    } else {
      return false;
    }
  }

}
