/**
 * Copyright (C) 2008 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.facade.runtime.impl;

import org.ow2.orchestra.facade.runtime.ActivityFullInstance;
import org.ow2.orchestra.facade.runtime.ProcessFullInstance;
import org.ow2.orchestra.facade.uuid.ActivityInstanceUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;

/**
 *
 * @author Gael Lalire
 */
public class ProcessFullInstanceImpl extends ProcessInstanceImpl implements ProcessFullInstance {

  private static final long serialVersionUID = 2606705701938051916L;

  protected ActivityFullInstance enclosedActivity;

  // for persistence
  protected long dbid;

  protected ProcessFullInstanceImpl() {
    super();
  }

  public ProcessFullInstanceImpl(final ProcessInstanceUUID uuid,
      final ProcessDefinitionUUID processUUID) {
    super(uuid, processUUID);
    this.enclosedActivity = null;
  }

  public ActivityFullInstance getEnclosedActivity() {
    return this.enclosedActivity;
  }

  public void setEnclosedActivity(final ActivityFullInstance enclosedActivity) {
    this.enclosedActivity = enclosedActivity;
    this.enclosedActivityUUID = enclosedActivity.getUUID();
  }

  @Override
  public ActivityInstanceUUID getEnclosedActivityUUID() {
    if (enclosedActivityUUID == null) {
      if (enclosedActivity == null) {
        return null;
      } else {
        enclosedActivityUUID = enclosedActivity.getUUID();
      }
    }
    return enclosedActivityUUID;
  }

  @Override
  public int hashCode() {
    return super.hashCode();
  }

  @Override
  public boolean equals(final Object obj) {
    if (obj == null) {
      return false;
    }
    if (!(obj.getClass().equals(this.getClass()))) {
      return false;
    }
    if (!super.equals(obj)) {
      return false;
    }
    ProcessFullInstanceImpl other = (ProcessFullInstanceImpl) obj;
    if (enclosedActivity == null) {
      if (other.enclosedActivity != null) {
        return false;
      }
    } else {
      if (!enclosedActivity.equals(other.enclosedActivity)) {
        return false;
      }
    }
    return dbid == other.dbid;
  }

}
