/**
 * Copyright (C) 2008 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.persistence.db;

import org.ow2.novabpm.util.Misc;
import org.ow2.orchestra.facade.data.ActivityInstanceData;
import org.ow2.orchestra.facade.data.ProcessInstanceData;
import org.ow2.orchestra.facade.runtime.ActivityFullInstance;
import org.ow2.orchestra.facade.runtime.ProcessFullInstance;
import org.ow2.orchestra.facade.runtime.impl.ActivityFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.impl.ProcessFullInstanceImpl;
import org.ow2.orchestra.facade.uuid.ActivityInstanceUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;
import org.ow2.orchestra.persistence.QuerierDbSession;
import org.ow2.orchestra.services.Recorder;

/**
 * 
 * @author Gael Lalire
 */
public class DbJournal extends AbstractDbQuerier implements Recorder {

  public DbJournal(final QuerierDbSession dbSession) {
    super(dbSession);
  }

  public void recordInstanceStarted(final ProcessInstanceData processInstanceData) {
    final ProcessDefinitionUUID processUUID = processInstanceData.getProcessDefinitionData().getUUID();
    final ProcessInstanceUUID instanceUUID = processInstanceData.getUUID();
    Misc.checkArgsNotNull(processUUID, instanceUUID);

    final ProcessFullInstance processFullInstance = new ProcessFullInstanceImpl(instanceUUID, processUUID);
    addProcessInstance(processFullInstance);
    getQuerierDbSession().save(processFullInstance);
  }

  public void recordInstanceEnded(final ProcessInstanceData processInstanceData) {
    final ProcessInstanceUUID instanceUUID = processInstanceData.getUUID();
    Misc.checkArgsNotNull(instanceUUID);

    final ProcessFullInstance processFullInstance = getProcessInstance(instanceUUID);
    processFullInstance.end();
  }

  public void recordActivityStarted(final ActivityInstanceData activityInstanceData) {
    final ProcessDefinitionUUID processUUID = activityInstanceData.getActivityDefinitionData().getProcessDefinitionData()
        .getUUID();
    final ProcessInstanceUUID instanceUUID = activityInstanceData.getProcessInstanceData().getUUID();
    final ActivityInstanceUUID activityUUID = activityInstanceData.getUUID();
    final ActivityInstanceUUID encloserInstanceUUID = activityInstanceData.getEncloserUUID();
    Misc.checkArgsNotNull(processUUID, instanceUUID, activityUUID);

    final ActivityFullInstance activityFullInstance = new ActivityFullInstanceImpl(activityUUID, processUUID, instanceUUID);

    addActivityInstance(activityFullInstance);

    if (encloserInstanceUUID == null) {
      // process direct child
      final ProcessFullInstance processFullInstance = getProcessInstance(instanceUUID);
      processFullInstance.setEnclosedActivity(activityFullInstance);
    } else if (encloserInstanceUUID.equals(activityFullInstance.getUUID())) {
      Misc.unreachableStatement("EncloserUUID (" + encloserInstanceUUID
          + ") cannot have the same UUID that the activityFullInstance (" + activityFullInstance.getUUID());
    } else {
      // parent activity
      final ActivityFullInstance encloserActivityFullInstance = getActivityInstance(encloserInstanceUUID);
      encloserActivityFullInstance.addEnclosedActivity(activityFullInstance);
    }
  }

  public void recordActivityEnded(final ActivityInstanceData activityInstanceData) {
    final ActivityInstanceUUID activityUUID = activityInstanceData.getUUID();
    final ActivityFullInstance activityFullInstance = getActivityInstance(activityUUID);
    activityFullInstance.end();
  }

  public void remove(ProcessFullInstance processFullInstance) {
    Misc.checkArgsNotNull(processFullInstance);
    getQuerierDbSession().delete(processFullInstance);
  }

}
