/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.persistence.db;

import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.logging.Logger;

import org.hibernate.FlushMode;
import org.hibernate.Query;
import org.hibernate.Session;
import org.jbpm.pvm.internal.hibernate.HibernatePvmDbSession;
import org.ow2.orchestra.facade.runtime.ActivityFullInstance;
import org.ow2.orchestra.facade.runtime.ActivityState;
import org.ow2.orchestra.facade.runtime.ProcessFullInstance;
import org.ow2.orchestra.facade.runtime.ProcessState;
import org.ow2.orchestra.facade.uuid.ActivityInstanceUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;
import org.ow2.orchestra.persistence.QuerierDbSession;

/**
 *
 * @author Gael Lalire
 */
public class QuerierDbSessionImpl extends HibernatePvmDbSession implements QuerierDbSession {

  private static final Logger LOG = Logger.getLogger(QuerierDbSessionImpl.class.getName());

  public QuerierDbSessionImpl(final Session session) {
    setSession(session);
    session.setFlushMode(FlushMode.COMMIT);
  }

  public ActivityFullInstance getActivityInstance(final ActivityInstanceUUID activityInstanceUUID) {
    Query query = getSession().getNamedQuery("findActivityInstanceByUUID");
    query.setString("uuid", activityInstanceUUID.toString());
    query.setMaxResults(1);
    return (ActivityFullInstance) query.uniqueResult();
  }

  public ProcessFullInstance getProcessInstance(final ProcessInstanceUUID processInstanceUUID) {
    Query query = getSession().getNamedQuery("findProcessInstanceByUUID");
    query.setString("uuid", processInstanceUUID.toString());
    query.setMaxResults(1);
    return (ProcessFullInstance) query.uniqueResult();
  }

  public Set<ActivityFullInstance> findActivityInstance(
      final ProcessInstanceUUID processUUID) {
    Query query = getSession().getNamedQuery("findActivityInstancesWithProcessUUID");
    query.setString("processInstanceUUID", processUUID.toString());
    List<ActivityFullInstance> results = query.list();
    if (results != null) {
      return new HashSet<ActivityFullInstance>(results);
    } else {
      return new HashSet<ActivityFullInstance>();
    }
  }

  public Set<ActivityFullInstance> findActivityInstance(
      final ProcessInstanceUUID processInstanceUUID, final ActivityState activityState) {
    Query query;
    switch (activityState) {
      case RUNNING:
        query = getSession().getNamedQuery(
            "findActivityInstancesWithProcessUUIDAndActive");
        break;
      case RUNNED:
        query = getSession().getNamedQuery(
            "findActivityInstancesWithProcessUUIDAndInactive");
        break;
      default:
        query = getSession().getNamedQuery("findActivityInstancesWithProcessUUID");
    }
    query.setString("processInstanceUUID", processInstanceUUID.toString());
    List<ActivityFullInstance> results = query.list();
    if (results != null) {
      return new HashSet<ActivityFullInstance>(results);
    } else {
      return new HashSet<ActivityFullInstance>();
    }
  }

  public Set<ProcessFullInstance> findProcessInstance() {
    Query query = getSession().getNamedQuery("findProcessInstances");
    List<ProcessFullInstance> results = query.list();
    if (results != null) {
      return new HashSet<ProcessFullInstance>(results);
    } else {
      return new HashSet<ProcessFullInstance>();
    }
  }

  public Set<ProcessFullInstance> findProcessInstance(
      final ProcessDefinitionUUID processDefinitionUUID) {
    Query query = getSession().getNamedQuery("findProcessInstancesWithProcessDefinitionUUID");
    query.setString("processDefinitionUUID", processDefinitionUUID.toString());
    List<ProcessFullInstance> results = query.list();
    if (results != null) {
      return new HashSet<ProcessFullInstance>(results);
    } else {
      return new HashSet<ProcessFullInstance>();
    }
  }

  public Set<ProcessFullInstance> findProcessInstance(
      final ProcessDefinitionUUID processDefinitionUUID, final ProcessState processState) {
    Query query;
    switch (processState) {
      case RUNNING:
        query = getSession().getNamedQuery(
            "findProcessInstancesWithProcessDefinitionUUIDAndActive");
        break;
      case RUNNED:
        query = getSession().getNamedQuery(
            "findProcessInstancesWithProcessDefinitionUUIDAndInactive");
        break;
      default:
        query = getSession().getNamedQuery(
            "findProcessInstancesWithProcessDefinitionUUID");
    }
    query.setString("processDefinitionUUID", processDefinitionUUID.toString());
    List<ProcessFullInstance> results = query.list();
    if (results != null) {
      return new HashSet<ProcessFullInstance>(results);
    } else {
      return new HashSet<ProcessFullInstance>();
    }
  }

  public Set<ProcessFullInstance> findProcessInstance(final ProcessState processState) {
    Query query;
    switch (processState) {
      case RUNNING:
        query = getSession().getNamedQuery("findProcessInstancesWithActive");
        break;
      case RUNNED:
        query = getSession().getNamedQuery("findProcessInstancesWithInactive");
        break;
      default:
        query = getSession().getNamedQuery("findProcessInstances");
    }
    List<ProcessFullInstance> results = query.list();
    if (results != null) {
      return new HashSet<ProcessFullInstance>(results);
    } else {
      return new HashSet<ProcessFullInstance>();
    }
  }

}


