/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.services;

import java.util.Set;

import org.ow2.orchestra.facade.exception.InstanceNotFoundException;
import org.ow2.orchestra.facade.runtime.ActivityFullInstance;
import org.ow2.orchestra.facade.runtime.ActivityState;
import org.ow2.orchestra.facade.runtime.ProcessFullInstance;
import org.ow2.orchestra.facade.runtime.ProcessState;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;

/**
 * A Querier is responsible for getting <strong>runtime</strong> instances
 * informations such as currently running instances and activities.
 *
 * @author Alfonso Fuca
 *
 */

public interface Querier {

  String DEFAULT_KEY = "queryList";

  /**
   * @param processInstanceUUID a process instance uuid
   * @return the process instance which matches uuid
   * @throws InstanceNotFoundException if this instance cannot be found
   */
  ProcessFullInstance getProcessInstance(final ProcessInstanceUUID processInstanceUUID);

  /**
   * Get informations about activity instances
   *
   * @param processUUID the uuid of the process instance
   * @return a set of activity instances corresponding to processUUID
   */
  Set<ActivityFullInstance> findActivityInstances(ProcessInstanceUUID processUUID);

  /**
   * Get informations about activity instances
   *
   * @param processUUID the uuid of the process instance
   * @param activityState the state of activities
   * @return a set of activity instances corresponding to processUUID and state
   *         'active'
   */
  Set<ActivityFullInstance> findActivityInstances(ProcessInstanceUUID processUUID, ActivityState activityState);

  /**
   * Get informations about process instances
   *
   * @return a set of all process instances
   */
  Set<ProcessFullInstance> findProcessInstances();

  /**
   * Get informations about process instances
   *
   * @param processDefinitionUUID the uuid of the process definition
   * @return a set of process instances corresponding to processDefinitionUUID
   */
  Set<ProcessFullInstance> findProcessInstances(ProcessDefinitionUUID processDefinitionUUID);

  /**
   * Get informations about process instances
   *
   * @param processDefinitionUUID the uuid of the process definition
   * @param processState the state of process instances
   * @return a set of process instances corresponding to processDefinitionUUID
   *         and state 'active'
   */
  Set<ProcessFullInstance> findProcessInstances(ProcessDefinitionUUID processDefinitionUUID, ProcessState processState);

  /**
   * Get informations about process instances
   *
   * @param processState state of process instances
   * @return a set of process instances corresponding to state 'active'
   */
  Set<ProcessFullInstance> findProcessInstances(ProcessState processState);

}
