/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.services;

import org.ow2.orchestra.facade.data.ActivityInstanceData;
import org.ow2.orchestra.facade.data.ProcessInstanceData;
import org.ow2.orchestra.facade.runtime.ProcessFullInstance;

/**
 * A Recorder is responsible for storing <strong>runtime</strong> instances
 * informations such as currently running instances and activities.
 * 
 * Usually, recorded records are available using queries on a {@link Journal}.
 * 
 * @author Alfonso Fuca
 * 
 */
public interface Recorder {

  String DEFAULT_KEY = "recorder";

  /**
   * Record informations about process instance started
   * 
   * @param processUUID the uuid of the process definition
   * @param instanceUUID the uuid of the process instance
   * @param processInstanceData data of the process instance
   */
  void recordInstanceStarted(ProcessInstanceData processInstanceData);

  /**
   * Record informations about process instance ended
   * 
   * @param instanceUUID the uuid of the process instance
   * @param processInstanceData data of the process instance
   */
  void recordInstanceEnded(ProcessInstanceData processInstanceData);

  /**
   * Record informations about activity instance started
   * 
   * @param processUUID the uuid of the process definition
   * @param instanceUUID the uuid of the process instance
   * @param activityUUID the uuid of the activity instance
   * @param encloserInstanceUUID the uuid of parent node execution it is null if
   *          we are the activity process
   * @param activityInstanceData data of the activity instance
   */
  void recordActivityStarted(ActivityInstanceData activityInstanceData);

  /**
   * Record informations about activity instance ended
   * 
   * @param activityUUID the uuid of the activity instance
   * @param activityInstanceData data of the activity instance
   */
  void recordActivityEnded(ActivityInstanceData activityInstanceData);

  /**
   * Generic method for removing a journal record.
   * 
   * @param processFullInstance the process to archive
   */
  void remove(ProcessFullInstance processFullInstance);

}
