/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.services.impl;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.ow2.orchestra.facade.runtime.ActivityFullInstance;
import org.ow2.orchestra.facade.runtime.ActivityState;
import org.ow2.orchestra.facade.runtime.ProcessFullInstance;
import org.ow2.orchestra.facade.runtime.ProcessState;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;
import org.ow2.orchestra.services.Querier;

/**
 * @author Guillaume Porcher
 *
 *         Chainer for Queriers : - for methods that returns only one object,
 *         search for the first matching object - for methods that returns a
 *         collection: search for all objects in all return queriers.
 */
public class QuerierChainer implements Querier {

  private final List<Querier> queriers;

  public QuerierChainer(final List<Querier> queriers) {
    this.queriers = queriers;
  }

  public Set<ActivityFullInstance> findActivityInstances(final ProcessInstanceUUID processUUID) {
    final Set<ActivityFullInstance> activityInsts = new HashSet<ActivityFullInstance>();
    for (final Querier querier : this.queriers) {
      final Set<ActivityFullInstance> tmp = querier.findActivityInstances(processUUID);
      if (tmp != null && !tmp.isEmpty()) {
        activityInsts.addAll(tmp);
      }
    }
    return activityInsts;
  }

  public Set<ActivityFullInstance> findActivityInstances(final ProcessInstanceUUID processUUID,
      final ActivityState activityState) {
    final Set<ActivityFullInstance> activityInsts = new HashSet<ActivityFullInstance>();
    for (final Querier querier : this.queriers) {
      final Set<ActivityFullInstance> tmp = querier.findActivityInstances(processUUID, activityState);
      if (tmp != null && !tmp.isEmpty()) {
        activityInsts.addAll(tmp);
      }
    }
    return activityInsts;
  }

  public Set<ProcessFullInstance> findProcessInstances() {
    final Set<ProcessFullInstance> processInsts = new HashSet<ProcessFullInstance>();
    for (final Querier querier : this.queriers) {
      final Set<ProcessFullInstance> tmp = querier.findProcessInstances();
      if (tmp != null && !tmp.isEmpty()) {
        processInsts.addAll(tmp);
      }
    }
    return processInsts;
  }

  public Set<ProcessFullInstance> findProcessInstances(final ProcessDefinitionUUID processDefinitionUUID) {
    final Set<ProcessFullInstance> processInsts = new HashSet<ProcessFullInstance>();
    for (final Querier querier : this.queriers) {
      final Set<ProcessFullInstance> tmp = querier.findProcessInstances(processDefinitionUUID);
      if (tmp != null && !tmp.isEmpty()) {
        processInsts.addAll(tmp);
      }
    }
    return processInsts;
  }

  public Set<ProcessFullInstance> findProcessInstances(final ProcessDefinitionUUID processDefinitionUUID,
      final ProcessState processState) {
    final Set<ProcessFullInstance> processInsts = new HashSet<ProcessFullInstance>();
    for (final Querier querier : this.queriers) {
      final Set<ProcessFullInstance> tmp = querier.findProcessInstances(processDefinitionUUID, processState);
      if (tmp != null && !tmp.isEmpty()) {
        processInsts.addAll(tmp);
      }
    }
    return processInsts;
  }

  public Set<ProcessFullInstance> findProcessInstances(final ProcessState processState) {
    final Set<ProcessFullInstance> processInsts = new HashSet<ProcessFullInstance>();
    for (final Querier querier : this.queriers) {
      final Set<ProcessFullInstance> tmp = querier.findProcessInstances(processState);
      if (tmp != null && !tmp.isEmpty()) {
        processInsts.addAll(tmp);
      }
    }
    return processInsts;
  }

  public ProcessFullInstance getProcessInstance(final ProcessInstanceUUID processInstanceUUID) {
    for (final Querier querier : this.queriers) {
      ProcessFullInstance processFullInstance =
        querier.getProcessInstance(processInstanceUUID);
      if (processFullInstance != null) {
        return processFullInstance;
      }
    }
    return null;
  }

}
